/**
 * @file
 * VuoList_VuoListPosition C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoListPosition_H
#define VuoList_VuoListPosition_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoListPosition VuoList_VuoListPosition
 * A list of @ref VuoListPosition elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoListPosition_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoListPosition elements.
 */
typedef const struct VuoList_VuoListPosition_struct { void *l; } * VuoList_VuoListPosition;
/// @}
#endif

/**
 * Creates a new list of @ref VuoListPosition elements.
 */
VuoList_VuoListPosition VuoListCreate_VuoListPosition(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoListPosition VuoListCopy_VuoListPosition(const VuoList_VuoListPosition list);

/**
 * Returns the @ref VuoListPosition at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoListPosition VuoListGetValue_VuoListPosition(const VuoList_VuoListPosition list, const unsigned long index);

/**
 * Changes the @ref VuoListPosition at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoListPosition(const VuoList_VuoListPosition list, const VuoListPosition value, const unsigned long index);

/**
 * Inserts the `VuoListPosition` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoListPosition(const VuoList_VuoListPosition list, const VuoListPosition value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoListPosition(VuoList_VuoListPosition list, const VuoListPosition value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoListPosition(VuoList_VuoListPosition list, const VuoListPosition value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoListPosition(VuoList_VuoListPosition list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoListPosition(VuoList_VuoListPosition list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoListPosition(VuoList_VuoListPosition list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoListPosition(VuoList_VuoListPosition list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoListPosition(VuoList_VuoListPosition list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoListPosition(VuoList_VuoListPosition list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoListPosition(VuoList_VuoListPosition list);

/**
 * Removes the `VuoListPosition` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoListPosition(VuoList_VuoListPosition list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoListPosition(const VuoList_VuoListPosition list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoListPosition VuoList_VuoListPosition_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoListPosition_getJson(const VuoList_VuoListPosition value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoListPosition_getSummary(const VuoList_VuoListPosition value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoListPosition VuoList_VuoListPosition_makeFromString(const char *str);
char * VuoList_VuoListPosition_getString(const VuoList_VuoListPosition value);
/// @}

/**
 * @}
 */

#endif
